"""
Static data resources for the mokume package.

This module provides access to organism data and histone protein information.
"""

from mokume.data.organisms import histones

__all__ = ["histones"]
